@@ -2,23 +2,11 @@
 Copyright 2007-2011 Free Software Foundation, Inc.
 This file is part of GNU Radio
 
-GNU Radio Companion is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
+SPDX-License-Identifier: GPL-2.0-or-later
 
-GNU Radio Companion is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
 
-from __future__ import absolute_import, print_function
 
 import logging
 import os
@@ -130,6 +118,7 @@ class Application(Gtk.Application):
 
         def flow_graph_update(fg=flow_graph):
             main.vars.update_gui(fg.blocks)
+            fg.namespace.clear()
             fg.update()
 
         ##################################################
@@ -204,6 +193,8 @@ class Application(Gtk.Application):
                 Actions.TOGGLE_FLOW_GRAPH_VAR_EDITOR,
                 Actions.TOGGLE_FLOW_GRAPH_VAR_EDITOR_SIDEBAR,
                 Actions.TOGGLE_HIDE_VARIABLES,
+                Actions.TOGGLE_SHOW_PARAMETER_EXPRESSION,
+                Actions.TOGGLE_SHOW_PARAMETER_EVALUATION,
                 Actions.TOGGLE_SHOW_BLOCK_IDS,
             ):
                 action.set_enabled(True)
@@ -235,7 +226,10 @@ class Application(Gtk.Application):
         elif action == Actions.NOTHING_SELECT:
             flow_graph.unselect()
         elif action == Actions.SELECT_ALL:
-            flow_graph.select_all()
+            if main.btwin.search_entry.has_focus():
+                main.btwin.search_entry.select_region(0, -1)
+            else:
+                flow_graph.select_all()
         ##################################################
         # Enable/Disable
         ##################################################
@@ -320,24 +314,24 @@ class Application(Gtk.Application):
             main.new_page()
             flow_graph = main.current_page.flow_graph
             Actions.BLOCK_PASTE()
-            coords = (x_min,y_min)
+            coords = (x_min, y_min)
             flow_graph.move_selected(coords)
 
             # Set flow graph to heir block type
-            top_block  = flow_graph.get_block("top_block")
+            top_block = flow_graph.get_block("top_block")
             top_block.params['generate_options'].set_value('hb')
 
             # this needs to be a unique name
             top_block.params['id'].set_value('new_hier')
 
             # Remove the default samp_rate variable block that is created
-            remove_me  = flow_graph.get_block("samp_rate")
+            remove_me = flow_graph.get_block("samp_rate")
             flow_graph.remove_element(remove_me)
 
             # Add the param blocks along the top of the window
             x_pos = 150
             for param in params:
-                param_id = flow_graph.add_new_block('parameter',(x_pos,10))
+                param_id = flow_graph.add_new_block('parameter', (x_pos, 10))
                 param_block = flow_graph.get_block(param_id)
                 param_block.params['id'].set_value(param)
                 x_pos = x_pos + 100
@@ -365,7 +359,7 @@ class Application(Gtk.Application):
                             pad_sink.dtype = source.dtype
 
                     # connect the pad to the proper sinks
-                    new_connection = flow_graph.connect(source,pad_sink)
+                    new_connection = flow_graph.connect(source, pad_sink)
 
                 elif pad['direction'] == 'source':
                     pad_id = flow_graph.add_new_block('pad_source', pad['coord'])
@@ -447,10 +441,12 @@ class Application(Gtk.Application):
             Dialogs.show_about(main, self.platform.config)
         elif action == Actions.HELP_WINDOW_DISPLAY:
             Dialogs.show_help(main)
+        elif action == Actions.GET_INVOLVED_WINDOW_DISPLAY:
+            Dialogs.show_get_involved(main)
         elif action == Actions.TYPES_WINDOW_DISPLAY:
             Dialogs.show_types(main)
         elif action == Actions.KEYBOARD_SHORTCUTS_WINDOW_DISPLAY:
-            Dialogs.show_keyboard_shortcuts(main)   
+            Dialogs.show_keyboard_shortcuts(main)
         elif action == Actions.ERRORS_WINDOW_DISPLAY:
             Dialogs.ErrorsDialog(main, flow_graph).run_and_destroy()
         elif action == Actions.TOGGLE_CONSOLE_WINDOW:
@@ -497,6 +493,14 @@ class Application(Gtk.Application):
             action.save_to_preferences()
             for page in main.get_pages():
                 flow_graph_update(page.flow_graph)
+        elif action == Actions.TOGGLE_SHOW_PARAMETER_EXPRESSION:
+            action.set_active(not action.get_active())
+            action.save_to_preferences()
+            flow_graph_update()
+        elif action == Actions.TOGGLE_SHOW_PARAMETER_EVALUATION:
+            action.set_active(not action.get_active())
+            action.save_to_preferences()
+            flow_graph_update()
         elif action == Actions.TOGGLE_HIDE_VARIABLES:
             action.set_active(not action.get_active())
             active = action.get_active()
@@ -551,13 +555,10 @@ class Application(Gtk.Application):
                     response = self.dialog.run()
                     if response in (Gtk.ResponseType.APPLY, Gtk.ResponseType.ACCEPT):
                         page.state_cache.save_new_state(flow_graph.export_data())
-                        ### Following  lines force an complete update of io ports
-                        n = page.state_cache.get_current_state()
-                        flow_graph.import_data(n)
+                        ### Following line forces a complete update of io ports
                         flow_graph_update()
-
                         page.saved = False
-                    else:  # restore the current state
+                    if response in (Gtk.ResponseType.REJECT, Gtk.ResponseType.ACCEPT):
                         n = page.state_cache.get_current_state()
                         flow_graph.import_data(n)
                         flow_graph_update()
@@ -617,11 +618,11 @@ class Application(Gtk.Application):
         elif action == Actions.FLOW_GRAPH_OPEN:
             file_paths = args[0] if args[0] else FileDialogs.OpenFlowGraph(main, page.file_path).run()
             if file_paths: # Open a new page for each file, show only the first
-                for i,file_path in enumerate(file_paths):
-                    main.new_page(file_path, show=(i==0))
+                for i, file_path in enumerate(file_paths):
+                    main.new_page(file_path, show=(i == 0))
                     self.config.add_recent_file(file_path)
                     main.tool_bar.refresh_submenus()
-                    #main.menu_bar.refresh_submenus()
+                    main.menu.refresh_submenus()
         elif action == Actions.FLOW_GRAPH_OPEN_QSS_THEME:
             file_paths = FileDialogs.OpenQSS(main, self.platform.config.install_prefix +
                                              '/share/gnuradio/themes/').run()
@@ -632,7 +633,9 @@ class Application(Gtk.Application):
         elif action == Actions.FLOW_GRAPH_OPEN_RECENT:
             file_path = str(args[0])[1:-1]
             main.new_page(file_path, show=True)
+            self.config.add_recent_file(file_path)
             main.tool_bar.refresh_submenus()
+            main.menu.refresh_submenus()
         elif action == Actions.FLOW_GRAPH_SAVE:
             #read-only or undefined file path, do save-as
             if page.get_read_only() or not page.file_path:
@@ -665,8 +668,7 @@ class Application(Gtk.Application):
                     page.saved = False
                 self.config.add_recent_file(file_path)
                 main.tool_bar.refresh_submenus()
-                #TODO
-                #main.menu_bar.refresh_submenus()
+                main.menu.refresh_submenus()
         elif action == Actions.FLOW_GRAPH_SAVE_COPY:
             try:
                 if not page.file_path:
@@ -708,6 +710,7 @@ class Application(Gtk.Application):
         # Gen/Exec/Stop
         ##################################################
         elif action == Actions.FLOW_GRAPH_GEN:
+            self.generator = None
             if not page.process:
                 if not page.saved or not page.file_path:
                     Actions.FLOW_GRAPH_SAVE()  # only save if file path missing or not saved
@@ -716,23 +719,27 @@ class Application(Gtk.Application):
                     try:
                         Messages.send_start_gen(generator.file_path)
                         generator.write()
+                        self.generator = generator
                     except Exception as e:
                         Messages.send_fail_gen(e)
-                else:
-                    self.generator = None
+
+
         elif action == Actions.FLOW_GRAPH_EXEC:
             if not page.process:
                 Actions.FLOW_GRAPH_GEN()
-                xterm = self.platform.config.xterm_executable
-                if self.config.xterm_missing() != xterm:
-                    if not os.path.exists(xterm):
-                        Dialogs.show_missing_xterm(main, xterm)
-                    self.config.xterm_missing(xterm)
-                if page.saved and page.file_path:
-                    Executor.ExecFlowGraphThread(
-                        flow_graph_page=page,
-                        xterm_executable=xterm,
-                        callback=self.update_exec_stop
+                if self.generator:
+                    xterm = self.platform.config.xterm_executable
+                    if self.config.xterm_missing() != xterm:
+                        if not os.path.exists(xterm):
+                            Dialogs.show_missing_xterm(main, xterm)
+                        self.config.xterm_missing(xterm)
+                    if page.saved and page.file_path:
+                        # Save config before execution
+                        self.config.save()
+                        Executor.ExecFlowGraphThread(
+                            flow_graph_page=page,
+                            xterm_executable=xterm,
+                            callback=self.update_exec_stop
                     )
         elif action == Actions.FLOW_GRAPH_KILL:
             if page.process:
@@ -753,7 +760,6 @@ class Application(Gtk.Application):
             main.update_pages()
 
         elif action == Actions.FIND_BLOCKS:
-            flow_graph.unselect()
             main.update_panel_visibility(main.BLOCKS, True)
             main.btwin.search_entry.show()
             main.btwin.search_entry.grab_focus()
